/*
Copyright 2008, Freie Universitaet Berlin. All rights reserved.

These sources implementing a version of the Distance Source Routing 
protocol (DSR)were developed at the Freie Universitt Berlin, for the
subject "Telematic Project: Wireless Embedded Systems".

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.
 
- Neither the name of Freie Universitaet Berlin (FUB) nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

This software is provided by FUB and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall FUB or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

This implementation was developed by the authors Joaquin Gonzalez and
Inas Sameer at the subject "Telematic Project: Wireless Embedded Systems"
during the Winter term 2008.

For documentation and questions please use the web site
http://scatterweb.mi.fu-berlin.de and the mailinglist
scatterweb@lists.spline.inf.fu-berlin.de (subscription via the Website).
Berlin, December 2008
*/

#include <string.h>
#include <ScatterWeb.System.h>
#include <ScatterWeb.CC1020.h>
#include "dsr.h"


//IMPLEMENTATION OF DSR PROTOCOL FUNCTIONS

//Function to initialize the Request Table
void InitReqTable(){
	int i;
	for (i=0; i<(MAX_ADDR-MIN_ADDR); i++){
		Req_table[i].dst = 0;
		Req_table[i].flag = 0;
	}
}

//Function to check if a given target node is in Request Table
bool isAddrInReqTable(netaddr_t dstnode){

	int i;

	//search node address in Request Table
	for (i=0; i<(MAX_ADDR-MIN_ADDR); i++){
		if (compAddr(Req_table[i].dst, dstnode)) return true;
	}

	//if address not found, node isn't in Request Table
	return false;

}


//Function to obtain an entry to a given target node in the Data Cache
bool Get_Data_Entry(netaddr_t dstnode, data_entry_t* data_entry){

	int i;
	bool found = false;
	for(i=0; i<Data_entries; i++){
		if (!found){
			//if data entry to dst node found, copy payload and delete entry
			if (compAddr(Data_Cache[i].dst, dstnode)){
				memcpy(data_entry->data, Data_Cache[i].data,sizeof(tcp_packet_t));
				Data_Cache[i].dst = 0;
				Data_Cache[i].reqcount = 0;
				found = true;
			}
		}
	}
	return found;
}


//Function to delete payload data
void Clear_Payload(payload_data data){
	int i;
	for(i=0; i<PayloadSize; i++){
		data[i] = 0;
	}
}

//Function to insert a new entry in the Data Cache
data_entry_t* Insert_Data_Entry(netaddr_t dstnode, payload_data data){
	int i, pos;
	pos = 0;
	bool found = false;

	//traverse Data Cache to look for a free entry (dst == 0)
	for(i=0; i<Data_entries; i++){
		if (!found){
			//if free entry found, insert entry
			if (compAddr(Data_Cache[i].dst, 0)){
				Data_Cache[i].dst = dstnode;
				//strcpy(,data);
				memcpy(Data_Cache[i].data, data, sizeof(tcp_packet_t));
				Data_Cache[i].reqcount = 0;
				if (SHOWDSR) Show_Data_Cache();
				pos = i;
				found = true;
			}
		}
	}

	if (!found) {
		return 0;
	}
	else {
		return &Data_Cache[pos];
	}
}

//Function to insert a given target node in Request Table
bool Insert_ReqTable_Entry(netaddr_t dstnode){

	int i;
	bool found = false;

	//traverse Request Table to look for a free entry (dst == 0)
	for(i=0; i<(MAX_ADDR-MIN_ADDR); i++){
		if (!found){
			//if free entry found, insert entry
			if (compAddr(Req_table[i].dst, 0)){
				Req_table[i].dst = dstnode;
				Req_table[i].flag = 1;
				found = true;
			}
		}
	}

	return found;
}

//Function to delete a given target node from Request Table
bool Delete_ReqTable_Entry(netaddr_t dstnode){

	int i;
	bool found = false;

	//traverse Request Table to look for a free entry (dst == 0)
	for(i=0; i<(MAX_ADDR-MIN_ADDR); i++){
		if (!found){
			//if entry found, entry
			if (compAddr(Req_table[i].dst, dstnode)){
				Req_table[i].dst = 0;
				Req_table[i].flag = 0;
				found = true;
			}
		}
	}
	return found;
}


//Function to initialize the Data Cache
void InitDataCache(){
	int i;
	for (i=0; i<Data_entries; i++){
		Data_Cache[i].dst = 0;
		Data_Cache[i].reqcount = 0;
		Clear_Payload(Data_Cache[i].data);
	}
}

//Function to check if node address belongs to defined local network address range
bool IsNtwAddr(netaddr_t node){

	if ((MIN_ADDR <= node) && (node <= MAX_ADDR)) {
		return true;
	}
	else 
		return false;
}

//Function to compare two node addresses
bool compAddr(netaddr_t node1, netaddr_t node2){

	if(node1 == node2) return true;
	else return false;
}

//Function to determine node position in Route Record sequence
int Look_up_Position(Route_record Route, netaddr_t node){

	int i, pos;

	pos=Net_diam;
	for(i=0; i<Net_diam; i++){
		if (compAddr(Route[i], node)) pos=i;
	}

	return pos;
}

//Function to determine if a node address is contained in a Route Record
bool isAddrInRoute(netaddr_t node, Route_record Route, int size){

	int i;

	//search node address in route record
	for (i=0; i<size; i++){
		if (compAddr(Route[i], node)) return true;
	}

	//if address not found, node isn't in route record
	return false;
}

//Function to add node address to ROUTE REQUEST Route List
bool Add_Hop_Req_Pckt(req_packet_t* req_pckt, netaddr_t node){
	if (req_pckt->pointer >= Net_diam){
		return false;
	}
	else {
		req_pckt->Route_rec[req_pckt->pointer] = node; // add node's address as next hop in list
		req_pckt->pointer ++;
		return true;
	}
}

//Function to show the content of a Request packet
void Show_Req_Pckt(req_packet_t* req_pckt){
	printf("Request packet content:\n");
	printf("Dest Node: %d\n",req_pckt->dest);
	printf("Pointer: %d\n",req_pckt->pointer);
	printf("TTL: %d\n",req_pckt->TTL);
	printf("Route record: ");
	int i;
	for (i=0; i<req_pckt->pointer; i++){
		printf("%d ", req_pckt->Route_rec[i]);
	}
	printf("\n\n");
}

//Function to show the content of a Response packet
void Show_Rsp_Pckt(rsp_packet_t* rsp_pckt){
	printf("Response packet content:\n");
	printf("Source Node: %d\n",rsp_pckt->source);
	printf("TTL: %d\n",rsp_pckt->TTL);
	printf("Route record: ");
	uint8_t i;
	
	for(i=0; i<=Look_up_Position(rsp_pckt->Route, rsp_pckt->source); i++){
		printf("%d ", rsp_pckt->Route[i]);
	}
	printf("\n\n");
}

//Function to show the content of a Data packet
void Show_Data_Pckt(data_packet_t* data_pckt){

	printf("Data packet content:\n");
	printf("Dest Node: %d\n",data_pckt->dest);
	printf("TTL: %d\n",data_pckt->TTL);
	
	printf("Route record: ");
	uint8_t i;
	for(i=0; i<=Look_up_Position(data_pckt->Route, data_pckt->dest); i++){
		printf("%d ", data_pckt->Route[i]);
	}
	printf("Payload: %s\n",data_pckt->data);
	printf("\n\n");

}

//Function to show the content of the Data Cache
void Show_Data_Cache(){

	printf("Data cache content:\n");
	uint8_t i;
	for(i=0; i<Data_entries; i++){
		printf("%d ", Data_Cache[i].dst);
	}
	printf("\n\n");

}

//Function to show Request Table
void Show_Req_Table(){
	printf("Req Table content:\n");
	uint8_t i;
	for(i=0; i<(MAX_ADDR-MIN_ADDR); i++){
		if (!compAddr(Req_table[i].dst, 0)){
			printf("Dst: %d Flag: %d\n", Req_table[i].dst, Req_table[i].flag);
		}
	}
	printf("\n\n");

}

//Data packet handling function
void Handle_Data_Pckt(data_packet_t* data_pckt){

	data_pckt->TTL--;

	if (data_pckt->TTL <= 0){
		if (SHOWDSR) printf("Max number of hops of DATA packet reached. Discarded\n");
	}
	else{
		
		//1st am I the final node? if yes, show data!
		if (compAddr(data_pckt->dest, Configuration.id)){
			//increase number of data packets received by layer 3 (DSR)
			RcvDataPcktCtr ++;
			if (SHOWDSR) printf("Received DSR data packet %d from %d\n",RcvDataPcktCtr,data_pckt->Route[0]);

			//transport header and payload data delivered to layer 4 reception handler 
			Handle_Tcp_Data_Pckt((tcp_packet_t*) data_pckt->data, data_pckt->Route[0]);

			Show_TCP_Packet((tcp_packet_t*) data_pckt->data);
			
		}
		else{//if i am intermediate node, look up next hop and forward data packet

			if (DEBUGDSR) {
				printf("I am an intermediate node. Forwarding data packet...\n");
				Show_Data_Pckt(data_pckt);
			}
			if (SHOWDSR) printf("Forwarding data packet from %d to %d over next hop %d\n", data_pckt->Route[0], data_pckt->dest, NextDsrDataHop(data_pckt->Route, Configuration.id));
			if (Forward_Data_Pckt(data_pckt, NextDsrDataHop(data_pckt->Route, Configuration.id))) printf("Data packet to %d over %d sent!\n", data_pckt->dest, NextDsrDataHop(data_pckt->Route, Configuration.id));

		}

	}

}

//Function to send Response packet from initiator to next hop in the Route Record
bool Send_Data_Pckt(Route_record Route, netaddr_t nextnode, netaddr_t dstnode){

	netpacket_send_args_t npsargs;
	data_packet_t data_pckt;
	data_entry_t data;

	//encapsulate DSR header in data field before sending it
	Init_Data_Pckt(&data_pckt);

	data_pckt.dest = dstnode;
	
	int i;
	for (i=0; i<Net_diam; i++){
		data_pckt.Route[i] = Route[i];
	}

	if (Get_Data_Entry(dstnode, &data)){
	
		memcpy(data_pckt.data,data.data,sizeof(tcp_packet_t));

		if (DEBUGDSR) Show_Data_Pckt(&data_pckt);

		// fill structure with default values
		Net_sendArgsInit( &npsargs );

		// fill network header custom values
		npsargs.netheader.to = nextnode;	// receiver's address
		npsargs.netheader.type = DSR_DATA_PACKET;	// application protocol
		npsargs.netheader.flags = 0;	// network layer config flags

		//encapsulate DSR header in packet data field before sending it
		npsargs.payload[0].buffer = (uint8_t*) &data_pckt;
		npsargs.payload[0].size = sizeof(data_pckt);

		if (DEBUGDSR) printf("Sending Data packet...\n");

		// queue packet for sending
		return Net_send( &npsargs );
	}
	else{
		if (SHOWDSR) printf("No data waiting for destination node %d !!", dstnode);
		return false;
	}
}

//Function to send all data packets in the send buffer refering to a given target node
void Send_Data_to_Dest(Route_record Route, netaddr_t nextnode, netaddr_t dstnode){

	int i;
	bool first, found; 
	first= true;
	found = false;
	data_entry_t* entry = 0;
	for(i=0; i<Data_entries; i++){
		//if data entry to dst node found, copy content and delete entry
		if (compAddr(Data_Cache[i].dst, dstnode)){
			if (Send_Data_Pckt(Route, nextnode, dstnode) && (SHOWDSR)) printf("Sent data packet to %d over %d!\n", dstnode, nextnode);
			//obtain pointer to the first entry to dest node 
			if (first){
				entry = &Data_Cache[i];
				first = false;
				found = true;
			}
		}
	}

	if (!found){
		if (SHOWDSR) printf("No entries currently available to dst node %d\n",dstnode);
	}
	else{
		if (SHOWDSR) printf("All data entries to dst node %d sent!!\n", dstnode);
		if (DEBUGDSR) Show_Data_Cache();
		//remove timer using first dest node entry
		Timers_remove(Restart_Route_Disc, entry);
	}
}

//Function to initialize a Data packet
void Init_Data_Pckt(data_packet_t* data_pckt){
	memset(data_pckt, 0, sizeof(data_packet_t)); // clear struct
	data_pckt->TTL=Net_diam; //set TTL to Network diameter
}


//Function to obtain next hop to a given node in a DATA packet Route Record
netaddr_t NextDsrDataHop(Route_record Route, netaddr_t node){
	
	return (Route[Look_up_Position(Route,node)+1]);

}

//Response packet handling function
void Handle_Rsp_Pckt(rsp_packet_t* rsp_pckt){

	if (SHOWDSR) Show_Rsp_Pckt(rsp_pckt);

	rsp_pckt->TTL--;

	if (rsp_pckt->TTL <= 0){
		if (SHOWDSR) printf("Max number of hops of RSP packet from %d to %d reached. Discarded\n", rsp_pckt->source, rsp_pckt->Route[0]);
	}
	else{
		
		//1st am I the final node? if yes, look up buffer
		if (compAddr(rsp_pckt->Route[0], Configuration.id)){

			if (SHOWDSR) printf("Received a response packet for me!!! Dst node : %d\n", rsp_pckt->source);
			if (DEBUGDSR) Show_Rsp_Pckt(rsp_pckt);
			
			if (SHOWDSR) printf("Sending data packet to %d over next hop %d", rsp_pckt->source, NextDsrDataHop(rsp_pckt->Route, Configuration.id));
			Send_Data_to_Dest(rsp_pckt->Route, NextDsrDataHop(rsp_pckt->Route, Configuration.id), rsp_pckt->source);
		
			//delete destination node from Request Table
			if( Delete_ReqTable_Entry(rsp_pckt->source) && (SHOWDSR)) printf("Destination node %d deleted from Req Table\n", rsp_pckt->source);
			if (SHOWDSR) Show_Req_Table();
		}
		else{//if i am intermediate node, look up next hop and forward rsp packet

			if (SHOWDSR) printf("I am an intermediate node. Forwarding rsp packet...\n");
			if (Forward_Rsp_Pckt( rsp_pckt , NextDsrRspHop(rsp_pckt->Route, Configuration.id)) && (SHOWDSR)) printf("RSP packet to %d forwarded!", rsp_pckt->Route[0]);
			if (DEBUGDSR) Show_Rsp_Pckt(rsp_pckt);

		}

	}
}

//Function to forward Data packet to next hop in message's Route Record
bool Forward_Data_Pckt(data_packet_t* data_pckt, netaddr_t nextnode){

	netpacket_send_args_t npsargs;
	data_packet_t data_pckt2;

	Init_Data_Pckt(&data_pckt2);

	data_pckt2.dest = data_pckt->dest;
	data_pckt2.TTL = data_pckt->TTL;

	//copy Route Record
	int i;
	for (i=0; i<Net_diam; i++){
		data_pckt2.Route[i] = data_pckt->Route[i];
	}

	memcpy(data_pckt2.data,data_pckt->data,sizeof(tcp_packet_t));

	if (DEBUGDSR) Show_Data_Pckt(&data_pckt2);

	// fill structure with default values
	Net_sendArgsInit( &npsargs );

	// fill network header custom values
	npsargs.netheader.to = nextnode;	// receiver's address
	npsargs.netheader.type = DSR_DATA_PACKET;	// application protocol
	npsargs.netheader.flags = 0;	// network layer config flags

	//encapsulate DSR RSP packet in data field before sending it
	npsargs.payload[0].buffer = (uint8_t*) &data_pckt2;
	npsargs.payload[0].size = sizeof(data_pckt2);

	if (DEBUGDSR) printf("Forwarding data packet...\n");

	// queue packet for sending
	return Net_send( &npsargs );

}

//Function to forward Response packet to next hop in message's Route Record
bool Forward_Rsp_Pckt(rsp_packet_t* rsp_pckt, netaddr_t nextnode){

	netpacket_send_args_t npsargs;
	rsp_packet_t rsp_pckt2;

	Init_Rsp_Pckt(&rsp_pckt2);
	rsp_pckt2.source = rsp_pckt->source;
	rsp_pckt2.TTL = rsp_pckt->TTL;

	//copy Route Record
	int i;
	for (i=0; i<Net_diam; i++){
		rsp_pckt2.Route[i] = rsp_pckt->Route[i];
	}

	if (DEBUGDSR) Show_Rsp_Pckt(&rsp_pckt2);

	// fill structure with default values
	Net_sendArgsInit( &npsargs );

	// fill network header custom values
	npsargs.netheader.to = nextnode;	// receiver's address
	npsargs.netheader.type = DSR_RESPONSE_PACKET;	// application protocol
	npsargs.netheader.flags = 0;	// network layer config flags

	//encapsulate DSR RSP packet in data field before sending it
	npsargs.payload[0].buffer = (uint8_t*) &rsp_pckt2;
	npsargs.payload[0].size = sizeof(rsp_pckt2);

	if (DEBUGDSR) printf("Forwarding rsp packet...\n");

	// queue packet for sending
	return Net_send( &npsargs );

}

//Function to obtain next hop to a given node in a RSP packet Route Record
netaddr_t NextDsrRspHop(Route_record Route, netaddr_t node){

	return (Route[Look_up_Position(Route,node)-1]);
}

//Function to initialize a Response packet
void Init_Rsp_Pckt(rsp_packet_t* rsp_pckt){
	memset(rsp_pckt, 0, sizeof(rsp_packet_t)); // clear struct
	rsp_pckt->source=Configuration.id;
	rsp_pckt->TTL=Net_diam; //set TTL to Network diameter
}

//Function to send Response packet from initiator back to next hop
bool Send_Rsp_Pckt(Route_record Route, netaddr_t nextnode){

	netpacket_send_args_t npsargs;
	rsp_packet_t rsp_pckt;

	//encapsulate DSR header in data field before sending it
	Init_Rsp_Pckt(&rsp_pckt);
	
	int i;
	for (i=0; i<Net_diam; i++){
		rsp_pckt.Route[i] = Route[i];
	}
	
	if (DEBUGDSR) Show_Rsp_Pckt(&rsp_pckt);

	// fill structure with default values
	Net_sendArgsInit( &npsargs );

	// fill network header custom values
	npsargs.netheader.to = nextnode;	// receiver's address
	npsargs.netheader.type = DSR_RESPONSE_PACKET;	// application protocol
	npsargs.netheader.flags = 0;	// network layer config flags

	//encapsulate DSR header in packet data field before sending it
	npsargs.payload[0].buffer = (uint8_t*) &rsp_pckt;
	npsargs.payload[0].size = sizeof(rsp_pckt);

	if (DEBUGDSR) printf("Sending Rsp packet...\n");

	// queue packet for sending
	return Net_send( &npsargs );

}

//Request packet handling function
void Handle_Req_Pckt(req_packet_t* req_pckt){
	
	//process request packet
	
	if (SHOWDSR) printf("REQ PACKET to: %d\n",req_pckt->dest);

	//update TTL and check if it has reached 0
	req_pckt->TTL--;

	if (req_pckt->TTL <= 0){
		if (SHOWDSR) printf("Max number of hops of REQ packet reached. Discarded\n");
	}
	else{
	//1st -> am I the destination node? -> YES -> send Response Packet
	if (compAddr(req_pckt->dest, Configuration.id)){
		if (!Add_Hop_Req_Pckt( req_pckt, Configuration.id)){
			if (SHOWDSR) printf("Route record limit size reached. REQ Packet discarded\n");
		}
		else{
			if (SHOWDSR) printf("Send response packet to %d\n", req_pckt->Route_rec[0]);
			//send back rsp packet to previous hop
			if (Send_Rsp_Pckt(req_pckt->Route_rec, NextDsrRspHop(req_pckt->Route_rec, Configuration.id)) && (SHOWDSR)) printf("RSP packet sent!\n");
			if (DEBUGDSR) Show_Req_Pckt(req_pckt);
		}
	}
	else if (!isAddrInRoute(Configuration.id,req_pckt->Route_rec,req_pckt->pointer)){//2nd -> NO -> check if my address is already in route
		//my address is NOT in route -> insert own address, update pointer and resend Req Packet
		if (!Add_Hop_Req_Pckt( req_pckt, Configuration.id)){
			if (SHOWDSR) printf("Route record limit size reached. REQ Packet discarded\n");
		}
		else{
			if (SHOWDSR) printf("I am not dest node. Forward REQ packet \n");
			if (Forward_Req_Pckt( req_pckt )) printf("REQ packet sent!\n");
			if (DEBUGDSR) Show_Req_Pckt(req_pckt);
		}
	}//if yes, discard request packet
	else{
		if (SHOWDSR) printf("I am already in the route list. REQ packet discarded\n\n");

		if (DEBUGDSR) Show_Req_Pckt(req_pckt);
	}
   }
}

//Function to forward Request packet to next hop in message's Route Record
bool Forward_Req_Pckt(req_packet_t* req_pckt){

	netpacket_send_args_t npsargs;
	req_packet_t req_pckt2;

	//encapsulate DSR header in data field before sending it
	Init_Req_Pckt(&req_pckt2);
	req_pckt2.dest = req_pckt->dest;
	req_pckt2.pointer = req_pckt->pointer;
	req_pckt2.TTL = req_pckt->TTL;

	int i;
	for (i=0; i<Net_diam; i++){
		req_pckt2.Route_rec[i] = req_pckt->Route_rec[i];
	}

	Show_Req_Pckt(req_pckt);

	// fill structure with default values
	Net_sendArgsInit( &npsargs );

	// fill network header custom values
	npsargs.netheader.to = LOCAL_BROADCAST;	// receiver's address
	npsargs.netheader.type = DSR_REQUEST_PACKET;	// application protocol
	npsargs.netheader.flags = 0;	// network layer config flags

	//encapsulate DSR header in packet data field before sending it
	npsargs.payload[0].buffer = (uint8_t*) &req_pckt2;
	npsargs.payload[0].size = sizeof(req_pckt2);

	if (SHOWDSR) printf("Sending Req packet...\n");

	// queue packet for sending
	return Net_send( &npsargs );
}

//Function to initialize a Request packet
void Init_Req_Pckt(req_packet_t* req_pckt)
{
	memset(req_pckt, 0, sizeof(req_packet_t)); // clear struct
	req_pckt->Route_rec[0] = Configuration.id; // this node's address as first in list
	req_pckt->pointer=1;
	req_pckt->TTL = Net_diam; //set TTL to Network diameter
}

//Function to delete all Data Buffer entries refered to the provided target node
void Delete_Data_Entries(netaddr_t dstnode){
	int i;

	for(i=0; i<Data_entries; i++){
		if (compAddr(Data_Cache[i].dst, dstnode)){
			Data_Cache[i].dst = 0;
			Clear_Payload(Data_Cache[i].data);
			Data_Cache[i].reqcount = 0;
		}
	}
}

//Function to start Route Discovery Process
bool Start_Route_Discovery(netaddr_t node) {
	netpacket_send_args_t npsargs;
	req_packet_t req_pckt;

	// fill structure with default values
	Net_sendArgsInit( &npsargs );

	// fill network header custom values
	npsargs.netheader.to = LOCAL_BROADCAST;	// receiver's address
	npsargs.netheader.type = DSR_REQUEST_PACKET;	// application protocol
	npsargs.netheader.flags = 0;	// network layer config flags

	//encapsulate DSR header in data field before sending it
	Init_Req_Pckt(&req_pckt);
	req_pckt.dest = node;

	npsargs.payload[0].buffer = (uint8_t*) &req_pckt;
	npsargs.payload[0].size = sizeof(req_pckt);

	// queue packet for sending
	return Net_send( &npsargs );
}

//Function to restart Route Discovery Process
void Restart_Route_Disc (void *datap){

	data_entry_t* e = (data_entry_t*) datap;

	e->reqcount++;

	//check if maximum number of retransmissions reached
	if (e->reqcount >3){
		if (SHOWDSR) printf("Maximum re-sending times reached. Erase data entries to dst node %d\n",e->dst);
		Delete_Data_Entries(e->dst);
		Delete_ReqTable_Entry(e->dst);
	}else{
		if (Start_Route_Discovery(e->dst)) printf("New route Discovery (%d) to dest node %d\n",e->reqcount, e->dst);
		Timers_add(1024*3, Restart_Route_Disc, e);
	}

}

//Function to send transport header and payload data using the DSR protocol
bool SendDSR(netaddr_t dstnode, payload_data data){

	data_entry_t* entry;

	if (SHOWDSR) Show_TCP_Packet((tcp_packet_t*)data);

	//increment number of packets to be sent by layer 3 protocol
	SentDataPcktCtr++;

	//insert data entry in data cache
	entry = Insert_Data_Entry(dstnode, data);
		
	if (entry != 0){

		if (DEBUGDSR){ 
			printf("Entry created in data cache\n");
			Show_Data_Cache();
		}

		//check if dest node is already in Data Buffer
		if(isAddrInReqTable(dstnode)){
			if (SHOWDSR) {
				printf("Dst node already in Req Table. No Route Discovery initialized\n");
				Show_Req_Table();
			}
		}
		else{
			
			Insert_ReqTable_Entry(dstnode);
			if (SHOWDSR) {
				printf("Dst node isn't in Req Table. Start Route Discovery\n");
				Show_Req_Table();
			}
			Start_Route_Discovery((netaddr_t)dstnode);
			Timers_add(1024*RTODSR, Restart_Route_Disc, entry);
		}
		return true;
	}
	else{//if no space available in data cache, discard entry
		printf("ERROR: no space available in data cache!\n");
		return false;
	}

}
